<?php

namespace App\Http\Controllers;

use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\ValidationException;
use App\Helpers\ApiResponseHelper;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;

class AuthController extends Controller
{
    // Register a new user
    public function register(Request $request)
    {
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'password' => 'required|string|min:6|confirmed',
        ]);

        $user = User::create([
            'name' => $validatedData['name'],
            'email' => $validatedData['email'],
            'password' => Hash::make($validatedData['password']),
        ]);

        return ApiResponseHelper::success($user, 'User registered successfully', 201);
    }

    /**
     * Login a user using external API authentication, save user, and fetch property rooms.
     */
    public function login(Request $request)
    {
        $validatedData = $request->validate([
            'email' => 'required|string',
            'password' => 'required|string',
        ]);

        $authBaseUrl = config('services.auth_api.base_url'); // External Auth API base URL
        $resbookBaseUrl = config('services.resbook_api.base_url'); // External Property API base URL

        // Step 1: External Authentication API Call
        $authResponse = Http::withHeaders([
            'Accept' => 'application/json',
        ])->post("{$authBaseUrl}/authenticate", [
            'username' => $validatedData['email'],
            'password' => $validatedData['password'],
        ]);

        if ($authResponse->failed()) {
            return ApiResponseHelper::error('Invalid credentials', 401);
        }

        $authData = $authResponse->json();

        // Step 2: Check if user exists in our database
        $user = User::where('email', $validatedData['email'])->first();

        if (! $user) {
            // Step 3: Create the user locally if not exists
            $user = User::create([
                'name' => $authData['user']['name'] ?? $validatedData['email'],
                'email' => $validatedData['email'],
                'password' => Hash::make($validatedData['password']), // Store hashed password
                'property_id' => $authData['user']['property_id'] ?? 6418, // Store property ID if available
            ]);
        }

        // Step 4: Generate API Token
        $token = $user->createToken('auth_token')->plainTextToken;
        $property_id = $user->property_id ?? 6418;

        $user->property_id = $property_id;
        $user->save();

        // Step 5: Fetch Property Rooms using Basic Auth
        $propertyResponse = Http::withBasicAuth($validatedData['email'], $validatedData['password'])
            ->get("{$resbookBaseUrl}/stock-units/stock-units", [
                'property_id' => $property_id, // Use dynamic property ID
            ]);

        if ($propertyResponse->failed()) {
            return ApiResponseHelper::error('Failed to fetch property rooms', 500);
        }

        $propertyRooms = $propertyResponse->json();

        // Step 6: Process Rooms (Insert, Update, Delete)
        $fetchedRoomIds = []; // Track fetched room IDs

        foreach ($propertyRooms as $room) {
            $fetchedRoomIds[] = $room['stock_unit_id']; // Store fetched room ID

            // Check if the room already exists for this property
            $existingRoom = DB::table('property_rooms')
                ->where('property_id', $property_id)
                ->where('stock_unit_id', $room['stock_unit_id'])
                ->first();

            if ($existingRoom) {
                // Update existing room
                DB::table('property_rooms')
                    ->where('id', $existingRoom->id)
                    ->update([
                        'stock_unit_name' => $room['stock_unit_name'],
                        'stock_unit_type' => $room['stock_unit_type'],
                        'classification_id' => $room['classification_id'],
                        'classification_name' => $room['classification_name'],
                        'classification_type' => $room['classification_type'],
                        'updated_at' => now(),
                    ]);
            } else {
                // Insert new room
                DB::table('property_rooms')->insert([
                    'id' => $room['stock_unit_id'], // Explicitly setting 'id'
                    'property_id' => $property_id,
                    'room_id' => $room['stock_unit_id'], // Keeping the old reference for room_id
                    'stock_unit_name' => $room['stock_unit_name'],
                    'stock_unit_id' => $room['stock_unit_id'],
                    'stock_unit_type' => $room['stock_unit_type'],
                    'classification_id' => $room['classification_id'],
                    'classification_name' => $room['classification_name'],
                    'classification_type' => $room['classification_type'],
                    'created_at' => now(),
                    'updated_at' => now(),
                ]);
            }
        }

        // Step 7: Remove any rooms that were previously stored but are missing now
        DB::table('property_rooms')
            ->where('property_id', $property_id)
            ->whereNotIn('stock_unit_id', $fetchedRoomIds)
            ->delete();

        // Step 8: Return success response with user details
        return ApiResponseHelper::success([
            'token' => $token,
            'user' => $user,
        ], 'User logged in successfully and property rooms updated.', 200);
    }



    // Logout user and delete token
    public function logout(Request $request)
    {
        $request->user()->tokens()->delete();

        return ApiResponseHelper::success(null, 'Logged out successfully', 200);
    }
}
