<?php

namespace App\Http\Controllers;

use App\Models\Rate;
use Illuminate\Http\Request;
use App\Helpers\ApiResponseHelper;
use App\Models\RateDate;
use App\Models\Season;
use Carbon\Carbon;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;

class RatesController extends Controller
{
    /**
     * Get rates by property_id and rate_type.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function getRatesByPropertyId(Request $request)
    {
        $request->validate([
            'property_id' => 'required|integer',
            'rate_type' => 'required|in:BASE,SEASONAL',
        ]);

        $propertyId = $request->input('property_id');
        $rateType = $request->input('rate_type');

        // Get room IDs associated with the property_id from the property_rooms table
        $roomIds = DB::table('property_rooms')
            ->where('property_id', $propertyId)
            ->pluck('room_id')
            ->toArray();

        // Fetch rates based on room IDs and rate_type, include season if SEASONAL
        $rates = Rate::with(['dates'])
            ->when($rateType === 'SEASONAL', function ($query) {
                $query->with('season');
            })
            ->whereIn('room_id', $roomIds)
            ->where('rate_type', $rateType)
            ->get();

        // Function to find the next available rate if the current week's rate is null
        function getNextAvailableRate($rate, $startDate)
        {
            $nextAvailableRate = null;
            $nextDate = $startDate->copy();

            // Search for the next available rate up to 4 weeks ahead
            for ($i = 0; $i < 4; $i++) {
                $nextAvailableRate = $rate->dates->where('date', $nextDate->toDateString())->first()->rate ?? null;
                if ($nextAvailableRate !== null) {
                    break;
                }
                $nextDate->addWeek();
            }

            return $nextAvailableRate;
        }

        if ($rateType === 'SEASONAL') {
            // Group SEASONAL rates by season
            $seasonalRates = [];

            foreach ($rates as $rate) {
                $season = $rate->season;
                if (!$season) {
                    continue;
                }

                // Initialize the season entry if not already set
                if (!isset($seasonalRates[$season->id])) {
                    $seasonalRates[$season->id] = [
                        'season_id' => $season->id,
                        'season_name' => $season->name,
                        'start_date' => $season->start_date,
                        'end_date' => $season->end_date,
                        'rates' => []
                    ];
                }

                // Prepare weekly rates
                $currentWeekStart = Carbon::now()->startOfWeek();
                $weekdayRates = [];

                foreach (['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'] as $index => $day) {
                    $currentDate = $currentWeekStart->copy()->addDays($index);
                    $weekdayRates["{$day}_rate"] = $rate->dates->where('date', $currentDate->toDateString())->first()->rate ?? getNextAvailableRate($rate, $currentDate);
                }

                $seasonalRates[$season->id]['rates'][] = array_merge($rate->only([
                    'id',
                    'room_id',
                    'extra_adult_rate',
                    'child_rate',
                    'public_holiday_rate',
                    'base_guests',
                    'child_max_age',
                    'max_guests',
                ]), $weekdayRates);
            }

            return ApiResponseHelper::success(array_values($seasonalRates), 'Seasonal rates grouped by season fetched successfully.');
        }

        // Handle BASE rate_type response
        $baseRates = $rates->map(function ($rate) {
            $currentWeekStart = Carbon::now()->startOfWeek();
            $weekdayRates = [];

            foreach (['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'] as $index => $day) {
                $currentDate = $currentWeekStart->copy()->addDays($index);
                $weekdayRates["{$day}_rate"] = $rate->dates->where('date', $currentDate->toDateString())->first()->rate ?? getNextAvailableRate($rate, $currentDate);
            }

            return array_merge($rate->only([
                'id',
                'room_id',
                'extra_adult_rate',
                'child_rate',
                'public_holiday_rate',
                'base_guests',
                'child_max_age',
                'max_guests',
            ]), $weekdayRates);
        });

        return ApiResponseHelper::success($baseRates, 'Base rates fetched successfully.');
    }

    /**
     * Create or update multiple rates in bulk.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function createRate(Request $request)
    {
        $data = $request->validate([
            'rates' => 'required|array', // Ensure the input is an array of rate objects
            'rates.*.room_id' => 'required|integer',
            'rates.*.rate_type' => 'required|in:BASE,SEASONAL',
            'rates.*.season_id' => 'nullable|exists:seasons,id', // Required for SEASONAL rates
            'rates.*.start_date' => 'required_if:rates.*.rate_type,BASE|date', // Required if BASE
            'rates.*.end_date' => 'required_if:rates.*.rate_type,BASE|date|after_or_equal:rates.*.start_date', // Required if BASE
            'rates.*.extra_adult_rate' => 'nullable|numeric',
            'rates.*.child_rate' => 'nullable|numeric',
            'rates.*.public_holiday_rate' => 'nullable|numeric',
            'rates.*.base_guests' => 'required|integer',
            'rates.*.child_max_age' => 'required|integer',
            'rates.*.max_guests' => 'required|integer',
            'rates.*.monday_rate' => 'nullable|numeric',
            'rates.*.tuesday_rate' => 'nullable|numeric',
            'rates.*.wednesday_rate' => 'nullable|numeric',
            'rates.*.thursday_rate' => 'nullable|numeric',
            'rates.*.friday_rate' => 'nullable|numeric',
            'rates.*.saturday_rate' => 'nullable|numeric',
            'rates.*.sunday_rate' => 'nullable|numeric',
        ]);

        $createdOrUpdatedRates = [];

        foreach ($data['rates'] as $rateData) {
            // Step 1: Determine if the rate already exists
            $rate = Rate::where('room_id', $rateData['room_id'])
                ->where('rate_type', $rateData['rate_type'])
                ->when($rateData['rate_type'] === 'SEASONAL', function ($query) use ($rateData) {
                    $query->where('season_id', $rateData['season_id']);
                })
                ->first();

            if ($rate) {
                // Update the existing rate
                $rate->update(Arr::except($rateData, [
                    'monday_rate',
                    'tuesday_rate',
                    'wednesday_rate',
                    'thursday_rate',
                    'friday_rate',
                    'saturday_rate',
                    'sunday_rate',
                    'start_date',
                    'end_date' // Exclude these as they are handled separately
                ]));
            } else {
                // Create a new rate entry
                $rate = Rate::create(Arr::except($rateData, [
                    'monday_rate',
                    'tuesday_rate',
                    'wednesday_rate',
                    'thursday_rate',
                    'friday_rate',
                    'saturday_rate',
                    'sunday_rate',
                    'start_date',
                    'end_date' // Exclude these as they are handled separately
                ]));
            }

            // Step 2: Determine the date range for the rate
            if ($rateData['rate_type'] === 'BASE') {
                $startDate = !empty($rateData['start_date']) ? Carbon::parse($rateData['start_date']) : Carbon::now()->startOfMonth();
                $endDate = !empty($rateData['end_date']) ? Carbon::parse($rateData['end_date']) : Carbon::now()->endOfMonth();
            } else { // SEASONAL
                $season = Season::findOrFail($rateData['season_id']);
                $startDate = Carbon::parse($season->start_date);
                $endDate = Carbon::parse($season->end_date);
            }

            // Step 3: Prepare date-specific rates for the given range
            $dateRates = [];
            for ($date = $startDate->copy(); $date->lte($endDate); $date->addDay()) {
                $dayOfWeek = strtolower($date->format('l')) . '_rate';
                if (isset($rateData[$dayOfWeek])) {
                    $dateRates[] = [
                        'rates_id' => $rate->id,
                        'date' => $date->toDateString(),
                        'rate' => $rateData[$dayOfWeek],
                        'created_at' => now(),
                        'updated_at' => now(),
                    ];
                }
            }

            // Step 4: Remove existing RateDate entries for the same date range
            RateDate::where('rates_id', $rate->id)
                ->whereBetween('date', [$startDate, $endDate])
                ->delete();

            // Step 5: Bulk insert the new date-specific rates
            if (!empty($dateRates)) {
                RateDate::insert($dateRates);
            }

            $createdOrUpdatedRates[] = [
                'rate_id' => $rate->id,
                'room_id' => $rate->room_id,
                'rate_type' => $rate->rate_type,
                'start_date' => $startDate->toDateString(),
                'end_date' => $endDate->toDateString(),
            ];
        }

        return ApiResponseHelper::success($createdOrUpdatedRates, 'Rates created or updated successfully.');
    }


    /**
     * Get rate details by room ID and month, returning only date-specific rates.
     *
     * @param Request $request
     * @param int $room_id
     * @return \Illuminate\Http\JsonResponse
     */
    public function getRateByRoomId(Request $request, $room_id)
    {
        $data = $request->validate([
            'month' => 'nullable|date_format:Y-m', // Optional month parameter in YYYY-MM format
        ]);

        // Determine start and end dates based on the month parameter or default to the current month
        $startDate = isset($data['month'])
            ? Carbon::parse($data['month'])->startOfMonth()
            : Carbon::now()->startOfMonth();
        $endDate = $startDate->copy()->endOfMonth();

        // Fetch all rate types for the given room_id and month
        $baseRate = Rate::where('room_id', $room_id)
            ->where('rate_type', 'BASE')
            ->with(['dates' => function ($query) use ($startDate, $endDate) {
                $query->whereBetween('date', [$startDate, $endDate]);
            }])
            ->first();

        $seasonalRate = Rate::where('room_id', $room_id)
            ->where('rate_type', 'SEASONAL')
            ->with(['dates' => function ($query) use ($startDate, $endDate) {
                $query->whereBetween('date', [$startDate, $endDate]);
            }])
            ->first();

        $specialRate = Rate::where('room_id', $room_id)
            ->where('rate_type', 'SPECIAL')
            ->with(['dates' => function ($query) use ($startDate, $endDate) {
                $query->whereBetween('date', [$startDate, $endDate]);
            }])
            ->first();

        // Combine all rates by date with priority SPECIAL > SEASONAL > BASE
        $ratesByDate = [];

        if ($baseRate) {
            foreach ($baseRate->dates as $date) {
                $ratesByDate[$date->date] = [
                    'id' => $date->id,
                    'date' => $date->date,
                    'rate' => $date->rate,
                    'rate_type' => 'BASE',
                ];
            }
        }

        if ($seasonalRate) {
            foreach ($seasonalRate->dates as $date) {
                $ratesByDate[$date->date] = [
                    'id' => $date->id,
                    'date' => $date->date,
                    'rate' => $date->rate,
                    'rate_type' => 'SEASONAL',
                ];
            }
        }

        if ($specialRate) {
            foreach ($specialRate->dates as $date) {
                $ratesByDate[$date->date] = [
                    'id' => $date->id,
                    'date' => $date->date,
                    'rate' => $date->rate,
                    'rate_type' => 'SPECIAL',
                ];
            }
        }

        if (empty($ratesByDate)) {
            return ApiResponseHelper::notFound('No rates found for the specified room and month.');
        }

        // Sort by date
        ksort($ratesByDate);

        return ApiResponseHelper::success(array_values($ratesByDate), 'Date-specific rates fetched successfully.');
    }



    public function updateRateById(Request $request)
    {
        // Validate incoming data; only `rate_id` is required
        $data = $request->validate([
            'rate_id' => 'required|integer|exists:rates,id',
            'rate_type' => 'string|nullable',
            'extra_adult_rate' => 'numeric|nullable',
            'child_rate' => 'numeric|nullable',
            'public_holiday_rate' => 'numeric|nullable',
            'base_guests' => 'integer|nullable',
            'child_max_age' => 'integer|nullable',
            'max_guests' => 'integer|nullable',
            'season_id' => 'nullable|exists:seasons,id',
            'room_id' => 'integer|nullable',
            'class_id' => 'integer|nullable',
            'monday_rate' => 'numeric|nullable',
            'tuesday_rate' => 'numeric|nullable',
            'wednesday_rate' => 'numeric|nullable',
            'thursday_rate' => 'numeric|nullable',
            'friday_rate' => 'numeric|nullable',
            'saturday_rate' => 'numeric|nullable',
            'sunday_rate' => 'numeric|nullable'
        ]);

        // Find the rate entry to update
        $rate = Rate::findOrFail($data['rate_id']);

        // Update only the main rate fields that were provided
        $rate->update(Arr::except($data, ['rate_id', 'monday_rate', 'tuesday_rate', 'wednesday_rate', 'thursday_rate', 'friday_rate', 'saturday_rate', 'sunday_rate']));

        // Check if weekday rates were provided and should be updated
        $weekdayRates = Arr::only($data, [
            'monday_rate',
            'tuesday_rate',
            'wednesday_rate',
            'thursday_rate',
            'friday_rate',
            'saturday_rate',
            'sunday_rate'
        ]);

        if (!empty($weekdayRates)) {
            // Define dates for the current month
            $startOfMonth = Carbon::now()->startOfMonth();
            $endOfMonth = Carbon::now()->endOfMonth();

            // Loop through each day of the current month
            for ($date = $startOfMonth; $date->lte($endOfMonth); $date->addDay()) {
                $dayOfWeek = strtolower($date->format('l')); // e.g., 'monday', 'tuesday', etc.

                // If a rate was provided for this day, update or create it
                if (isset($weekdayRates["{$dayOfWeek}_rate"])) {
                    RateDate::updateOrCreate(
                        [
                            'rates_id' => $rate->id,
                            'date' => $date->toDateString()
                        ],
                        [
                            'rate' => $weekdayRates["{$dayOfWeek}_rate"],
                        ]
                    );
                }
            }
        }

        // Fetch updated rates in the desired format
        $updatedRate = $this->formatRateResponse($rate->id);

        return ApiResponseHelper::success($updatedRate, 'Rate updated successfully');
    }

    /**
     * Update rates for multiple specific dates.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function updateRateForDate(Request $request)
    {
        // Validate the request to ensure the payload is an array of objects
        $data = $request->validate([
            'rates' => 'required|array',
            'rates.*.rate_date_id' => 'required|integer|exists:rates_dates,id',
            'rates.*.rate' => 'required|numeric',
        ]);

        $updatedRates = [];

        // Process each rate update
        foreach ($data['rates'] as $rateUpdate) {
            // Find the RateDate entry by rate_date_id
            $rateDate = RateDate::findOrFail($rateUpdate['rate_date_id']);

            // Update the rate for the specific date
            $rateDate->update([
                'rate' => $rateUpdate['rate'],
            ]);

            $updatedRates[] = $rateDate;
        }

        return ApiResponseHelper::success($updatedRates, 'Rates updated successfully for the specified dates.');
    }
    private function formatRateResponse($rateId)
    {
        $rate = Rate::with('dates')->findOrFail($rateId);

        // Define the start and end of the current month
        $startOfMonth = Carbon::now()->startOfMonth();
        $endOfMonth = Carbon::now()->endOfMonth();

        // Check if rates are consistent by weekday across the entire month
        $isConsistentByWeekday = true;
        $weekdayRates = [];

        // Populate the weekdayRates array based on the first week’s data
        foreach (['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'] as $day) {
            $date = $startOfMonth->copy()->next($day);
            $weekdayRates["{$day}_rate"] = $rate->dates->where('date', $date->toDateString())->first()->rate ?? null;
        }

        // Verify if rates are consistent throughout the month
        foreach ($rate->dates as $dateEntry) {
            $dayOfWeek = strtolower(Carbon::parse($dateEntry->date)->format('l'));

            if ($weekdayRates["{$dayOfWeek}_rate"] !== $dateEntry->rate) {
                $isConsistentByWeekday = false;
                break;
            }
        }

        if ($isConsistentByWeekday) {
            // Return grouped by weekdays if consistent
            $weeklyRates = $weekdayRates;
        } else {
            // Return rates for the current week if they vary day-to-day
            $currentWeek = [
                'monday' => Carbon::now()->startOfWeek(),
                'tuesday' => Carbon::now()->startOfWeek()->addDays(1),
                'wednesday' => Carbon::now()->startOfWeek()->addDays(2),
                'thursday' => Carbon::now()->startOfWeek()->addDays(3),
                'friday' => Carbon::now()->startOfWeek()->addDays(4),
                'saturday' => Carbon::now()->startOfWeek()->addDays(5),
                'sunday' => Carbon::now()->startOfWeek()->addDays(6)
            ];

            $weeklyRates = [];
            foreach ($currentWeek as $day => $date) {
                $weeklyRates["{$day}_rate"] = $rate->dates->where('date', $date->toDateString())->first()->rate ?? null;
            }
        }

        // Combine rate details with weekly/daily-specific rates
        return array_merge(Arr::except($rate->toArray(), ['dates']), $weeklyRates);
    }
}
