<?php

namespace App\Http\Controllers;

use App\Helpers\ApiResponseHelper;
use App\Models\Season;
use App\Models\Rate;
use App\Models\RateDate;
use App\Models\RatesDate;
use Carbon\Carbon;
use Illuminate\Http\Request;

class SeasonsController extends Controller
{
    // 1. Create a Season
    /**
     * Create or update a season for a property.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function createOrUpdateSeason(Request $request)
    {
        // Step 1: Validate the input data
        $data = $request->validate([
            'season_id' => 'nullable|integer|exists:seasons,id', // Optional for updates
            'property_id' => 'required|integer',
            'name' => 'required|string|max:255',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
        ]);

        // Step 2: If season_id is provided, find the season, otherwise prepare for creation
        $season = null;
        if (!empty($data['season_id'])) {
            $season = Season::find($data['season_id']);
        }

        // Step 3: Check constraints (only for creation or when dates are updated in an existing season)
        if (!$season || $season->start_date !== $data['start_date'] || $season->end_date !== $data['end_date']) {
            // Ensure no more than 4 seasons exist for the property
            $seasonCount = Season::where('property_id', $data['property_id'])
                ->when($season, function ($query) use ($season) {
                    $query->where('id', '!=', $season->id); // Exclude current season from the count
                })
                ->count();

            if ($seasonCount >= 4) {
                return ApiResponseHelper::error('A maximum of 4 seasons is allowed per property.', 400);
            }

            // Check for overlapping seasons within the property
            $overlapCheck = Season::where('property_id', $data['property_id'])
                ->when($season, function ($query) use ($season) {
                    $query->where('id', '!=', $season->id); // Exclude the current season from the overlap check
                })
                ->where(function ($query) use ($data) {
                    $query->whereBetween('start_date', [$data['start_date'], $data['end_date']])
                        ->orWhereBetween('end_date', [$data['start_date'], $data['end_date']])
                        ->orWhere(function ($query) use ($data) {
                            $query->where('start_date', '<=', $data['start_date'])
                                ->where('end_date', '>=', $data['end_date']);
                        });
                })
                ->exists();

            if ($overlapCheck) {
                return ApiResponseHelper::error('Season dates overlap with an existing season.', 400);
            }
        }

        // Step 4: Create or update the season
        if ($season) {
            // Update the existing season
            $season->update($data);
            $message = 'Season updated successfully.';
        } else {
            // Create a new season
            $season = Season::create($data);
            $message = 'Season created successfully.';
        }

        return ApiResponseHelper::success($season, $message);
    }


    // Call `createRate` to set seasonal rates
    public function setSeasonalRates(Request $request, RatesController $ratesController)
    {
        // Set `rate_type` to `SEASONAL` before passing to `createRate`
        $request->merge(['rate_type' => 'SEASONAL']);

        return $ratesController->createRate($request);
    }
}
