<?php

namespace App\Http\Controllers;

use App\Models\Rate;
use App\Models\SpecialRate;
use App\Models\SpecialRateRule;
use App\Models\RateDate;
use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Helpers\ApiResponseHelper;
use Illuminate\Support\Facades\DB;

class SpecialRatesController extends Controller
{
    /**
     * Save or update a special rate and its rules.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function saveSpecialRate(Request $request)
    {
        $data = $request->validate([
            'room_id' => 'required|integer',
            'specialname' => 'required|string|max:255',
            'start_date' => 'required|date',
            'end_date' => 'required|date|after_or_equal:start_date',
            'message' => 'string',
            'rule.day_of_week' => 'required|array',
            'rule.day_of_week.*' => 'in:Mon,Tue,Wed,Thu,Fri,Sat,Sun',
            'rule.rate_modifier' => 'required|in:increase,decrease',
            'rule.modifier_type' => 'required|in:fixed,percentage',
            'rule.amount' => 'required|numeric',
        ]);

        // Step 1: Check if a SPECIAL rate exists for the room_id
        $rate = Rate::firstOrCreate(
            [
                'room_id' => $data['room_id'],
                'rate_type' => 'SPECIAL',
            ],
            [
                'base_guests' => 0,
                'child_max_age' => 0,
                'max_guests' => 0,
            ]
        );

        // Step 2: Check for an existing special rate and update it
        $specialRate = SpecialRate::where('rates_id', $rate->id)->first();

        if ($specialRate) {
            // Update the existing special rate
            $specialRate->update([
                'specialname' => $data['specialname'],
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'message' => $data['message']
            ]);

            // Delete old rules and replace them
            $specialRate->rule()->delete();
        } else {
            // Create a new special rate
            $specialRate = SpecialRate::create([
                'rates_id' => $rate->id,
                'specialname' => $data['specialname'],
                'start_date' => $data['start_date'],
                'end_date' => $data['end_date'],
                'message' => $data['message']

            ]);
        }

        // Step 3: Insert new rules into `special_rate_rules`
        foreach ($data['rule']['day_of_week'] as $day) {
            SpecialRateRule::create([
                'special_rate_id' => $specialRate->id,
                'day_of_week' => $day,
                'rate_modifier' => $data['rule']['rate_modifier'],
                'modifier_type' => $data['rule']['modifier_type'],
                'amount' => $data['rule']['amount'],
            ]);
        }

        // Step 4: Delete all existing RateDate entries for this rates_id
        RateDate::where('rates_id', $rate->id)->delete();

        // Step 5: Apply rules and create new RateDate entries
        $dateRange = Carbon::parse($data['start_date'])->daysUntil($data['end_date']);
        foreach ($dateRange as $date) {
            // Check if the current date matches any rule's day of the week
            if (in_array($date->format('D'), $data['rule']['day_of_week'])) {
                // Fetch base or seasonal rate for the date
                $baseRate = RateDate::where('date', $date->toDateString())
                    ->whereHas('rate', function ($query) use ($data) {
                        $query->where('room_id', $data['room_id'])
                            ->whereIn('rate_type', ['BASE', 'SEASONAL']);
                    })
                    ->value('rate') ?? 0;

                // Apply the modifier logic
                $calculatedRate = $baseRate;
                if ($data['rule']['modifier_type'] === 'fixed') {
                    $calculatedRate += $data['rule']['rate_modifier'] === 'increase'
                        ? $data['rule']['amount']
                        : -$data['rule']['amount'];
                } elseif ($data['rule']['modifier_type'] === 'percentage') {
                    $adjustment = ($baseRate * $data['rule']['amount']) / 100;
                    $calculatedRate += $data['rule']['rate_modifier'] === 'increase'
                        ? $adjustment
                        : -$adjustment;
                }

                // Insert or update rate in `rates_dates`
                RateDate::create([
                    'rates_id' => $specialRate->rates_id,
                    'date' => $date->toDateString(),
                    'rate' => $calculatedRate,
                ]);
            }
        }

        return ApiResponseHelper::success(
            ['special_rate' => $specialRate->load('rule')],
            'Special rate saved or updated successfully.'
        );
    }


    /**
     * Show all special rates by property_id.
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function getSpecialRatesByPropertyId(Request $request)
    {
        $data = $request->validate([
            'property_id' => 'required|integer',
        ]);

        $propertyId = $data['property_id'];

        // Step 1: Get room_ids associated with the property_id
        $roomIds = DB::table('property_rooms')
            ->where('property_id', $propertyId)
            ->pluck('room_id')
            ->toArray();

        if (empty($roomIds)) {
            return ApiResponseHelper::success([], 'No rooms found for the given property.');
        }

        // Step 2: Get rate IDs from the rates table where rate_type is SPECIAL
        $rateIds = DB::table('rates')
            ->whereIn('room_id', $roomIds)
            ->where('rate_type', 'SPECIAL')
            ->pluck('id')
            ->toArray();

        if (empty($rateIds)) {
            return ApiResponseHelper::success([], 'No special rates found for the given property.');
        }

        // Step 3: Fetch special rates using rate IDs
        $specialRates = SpecialRate::with(['rate', 'rule'])
            ->whereIn('rates_id', $rateIds)
            ->get();

        // Step 4: Format the response and include room_id and rules
        $response = $specialRates->map(function ($specialRate) {
            // Get the room ID associated with this rate
            $roomId = $specialRate->rate->room_id ?? 'All';

            // Format the rules
            $rules = $specialRate->rule->map(function ($rule) {
                return [
                    'day_of_week' => $rule->day_of_week,
                    'rate_modifier' => $rule->rate_modifier,
                    'modifier_type' => $rule->modifier_type,
                    'amount' => $rule->amount,
                ];
            });

            // Format the response structure
            return [
                'id' => $specialRate->id,
                'special_rate_title' => $specialRate->specialname,
                'room_id' => $roomId,
                'rooms' => is_array($roomId) ? implode(', ', $roomId) : $roomId,
                'dates' => $specialRate->start_date . ' - ' . $specialRate->end_date,
                'start_date' => $specialRate->start_date,
                'end_date' => $specialRate->end_date,
                'price_modifier' => $this->formatModifier($specialRate),
                'message' => $specialRate->message,
                'rules' => $rules,
            ];
        });

        return ApiResponseHelper::success($response, 'Special rates fetched successfully.');
    }


    /**
     * Delete a special rate and its related data, including the associated rate.
     *
     * @param Request $request
     * @param int $specialRateId
     * @return \Illuminate\Http\JsonResponse
     */
    public function deleteSpecialRate(Request $request, $specialRateId)
    {
        // Step 1: Find the special rate by its ID
        $specialRate = SpecialRate::find($specialRateId);

        if (!$specialRate) {
            return ApiResponseHelper::notFound('Special rate not found.');
        }

        // Step 2: Delete associated rules from `special_rate_rules`
        $specialRate->rule()->delete();

        // Step 3: Delete associated RateDate entries from `rates_dates`
        RateDate::where('rates_id', $specialRate->rates_id)->delete();


        // Step : Delete the special rate itself
        $specialRate->delete();

        // Step 5: Delete the associated rate entry from the `rates` table
        Rate::where('id', $specialRate->rates_id)->delete();



        return ApiResponseHelper::success([], 'Special rate and associated rate deleted successfully.');
    }



    /**
     * Format the price modifier.
     *
     * @param SpecialRate $specialRate
     * @return string
     */
    private function formatModifier($specialRate)
    {
        $rule = $specialRate->rule->first();
        if (!$rule) {
            return '';
        }

        $modifier = $rule->rate_modifier === 'increase' ? '+' : '-';
        $amount = $rule->modifier_type === 'fixed' ? '$' . $rule->amount : $rule->amount . '%';

        return $modifier . $amount;
    }
}
